/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;

import java.awt.event.*;
import java.util.*;

class RemoteSolverPanel 
    extends SolverPanel implements RemoteSyncListener
{
    /* states of the root BeaconConfigApp */
    private RemoteSyncEvent currentStates;
    private EventListener eventHandler;

    public RemoteSolverPanel(BeaconConfig bc, DrawingArea da, int closeness, 
			     int stddev, 
			     int enoughsamples,
			     EventListener e)
    {
	super(bc, da, closeness, stddev, enoughsamples);
	currentStates = new RemoteSyncEvent();
	currentStates.status = super.status.getText();
	currentStates.errmsg = super.errmsg.getText();
	eventHandler = e;
    }

    protected void changeStatus(String s)
    {
	super.changeStatus(s);
	//System.out.println("changeStatus: "+s);
	currentStates = new RemoteSyncEvent();
	currentStates.status = super.status.getText();

	if(eventHandler != null) // could be null when super is called within constructor
	    ((RemoteSyncListener) eventHandler).remoteSyncEvent(currentStates);
    }

    protected boolean errMsg(boolean beep, String s) {
	if(!super.errMsg(beep, s))
	    return false;

	//System.out.println("errMsg "+s);

	currentStates = new RemoteSyncEvent();
	currentStates.errmsg = super.errmsg.getText();

	if(eventHandler != null) // could be null when super is called within constructor
	    ((RemoteSyncListener) eventHandler).remoteSyncEvent(currentStates);

	return true;
    }
    

    private void setCurrentStates(RemoteSyncEvent cs)
    {
	Util.dbgPrintln("RemoteSolverPanel::setCurrentStates");
	currentStates = cs;

	//System.out.println("Client: s="+cs.status+" e="+cs.errmsg);
	
	if(cs.status != null)
	    super.status.setText(cs.status);
	if(cs.errmsg != null)
	    super.errmsg.setText(cs.errmsg);
    }

    // called by the Network Client in remote app
    public void remoteSyncEvent(RemoteSyncEvent e) {
	Util.dbgPrintln("RemoteControls::remoteSyncEvent");
	setCurrentStates(e);
    }

}
