/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.clientlib.parse;

import cricketdaemon.Util;
import cricketdaemon.clientlib.data.*;
import java.io.*;
import java.util.*;

/**
 * Parses subtypes contained within a "device_pos"ition type.
 *
 * @author Allen Miu
 */
class DevicePosParseHandlerMap extends ParseHandlerMap
{
    public static DevicePosParseHandlerMap instance = 
	new DevicePosParseHandlerMap();

    protected DevicePosParseHandlerMap()
    {
	super();
	register("algor", "check_datatype", "parse_algor");
	register("pos", "check_datatype", "parse_pos");
	register("array:dist_est", "check_datatype", 
		 "parse_adist_est");
    }

    public boolean check_datatype(Object o) 
    {
	return o instanceof CricketData;
    }

    public void parse_algor(Object dataStructure, String value)
	throws IOException 
    {
	CricketData c = (CricketData) dataStructure;
	c.setPosAlgor(value);
    }

    public void parse_pos(Object dataStructure, String value)
	throws IOException 
    {
	CricketData c = (CricketData) dataStructure;
	Position pos = new Position(-1, -1, -1);
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(pos,
			     sr, 
			     PosParseHandlerMap.instance);
	c.setDevicePosition(pos);
    }

    public void parse_adist_est(Object dataStructure, String value)
	throws IOException 
    {
	ArrayList calcBeaconDist = new ArrayList();
	CricketData cricketData = (CricketData) dataStructure;
	
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(calcBeaconDist,
			     sr, 
			     DistanceEstimateArrayParseHandlerMap.instance);

	cricketData.setCalcBeaconDist(calcBeaconDist);
	//PacketParseHandlerMap.parse_adist_est(dataStructure, value);
    }
}
